/*
 * Copyright (c) 2004-2006 Weswit s.r.l., Via Campanini, 6 - 20124 Milano, Italy.
 * All rights reserved.
 * www.lightstreamer.com
 *
 * This software is the confidential and proprietary information of
 * Weswit s.r.l.
 * You shall not disclose such Confidential Information and shall use it
 * only in accordance with the terms of the license agreement you entered
 * into with Weswit s.r.l.
 */

using System;
using System.Threading;
using System.Collections;

namespace Nms.Server.Test {

	/// <summary>
	/// Used by the Stock List Adapter to receive data from the simulated broadcast feed in an
	/// asynchronous way, through the OnEvent method.
	/// </summary>
	public interface IExternalFeedListener {
		
		/// <summary>
		/// Called by the feed for each update event occurrence on some stock.
		/// If isSnapshot is true, then the event contains a full snapshot,
		/// with the current values of all fields for the stock.
		/// </summary>
		void OnEvent(string itemName, IDictionary currentValues, bool isSnapshot);
	}

	/// <summary>
	/// Used by the Stock List Adapter, it simulates an external data feed that supplies quote values for all the
	/// stocks needed for the demo.
	/// </summary>
	public class ExternalFeedSimulator {

		/// <summary>
		/// Used to automatically generate the updates for the 30 stocks:
		/// mean and standard deviation of the times between consecutive
		/// updates for the same stock.
		/// </summary>
		private double [] _updateTimeMeans;		
		private double [] _updateTimeStdDevs;

		/// <summary>
		/// Used to generate the initial field values for the 30 stocks
		/// </summary>
		private int [] _equipStatus;		
		private string [] _equipNames;
		
		/// <summary>
		/// Used to keep the contexts of the 30 stocks.
		/// </summary>
		private IDictionary _statusGenerators;

		private IExternalFeedListener _listener;

		private IList _snapshotQueue;
		private Thread _snapshotSender;
		
		public ExternalFeedSimulator() {
			_statusGenerators= new Hashtable();
			_snapshotQueue= new ArrayList();

			_updateTimeMeans = new double [] {30000, 500, 3000 };
		
			_updateTimeStdDevs = new double [] {6000, 300, 1000 };

			_equipStatus = new int [] { 1, 2, 3 };
		

			_equipNames = new string [] {"EQUIPMENT 1", "EQUIPMENT 2", 
											"EQUIPMENT 3" };
		}

		/// <summary>
		/// Starts generating update events for the stocks. Sumulates attaching
		/// and reading from an external broadcast feed.
		/// </summary>
		public void Start() {
			if (_snapshotSender != null) return;

			for (int i = 0; i < 3; i++) {
				string itemName= "item" + (i + 1);
				ExternalFeedProducer myProducer = new ExternalFeedProducer(
				    itemName, 
					_equipStatus[i], 
					_updateTimeMeans[i], 
				    _updateTimeStdDevs[i], 
				    _equipNames[i] );
				
				_statusGenerators[itemName]= myProducer;
				myProducer.SetFeedListener(_listener);
				myProducer.Start();
			}

			_snapshotSender= new Thread(new ThreadStart(Run));
			_snapshotSender.Start();
		}

		private void Run() {
			IList snapshots= new ArrayList();
			do {
				lock (_snapshotQueue) {
					if (_snapshotQueue.Count == 0) 
						Monitor.Wait(_snapshotQueue);

					snapshots.Clear();
					while (_snapshotQueue.Count > 0) {
						ExternalFeedProducer myProducer= (ExternalFeedProducer) _snapshotQueue[0];
						snapshots.Add(myProducer);
						_snapshotQueue.RemoveAt(0);
					}
				}

				foreach (ExternalFeedProducer myProducer in snapshots) {
					_listener.OnEvent(myProducer.GetItemName(), myProducer.GetCurrentValues(true), true);
				}

			} while (true);
		}

		/// <summary>
		/// Sets an internal listener for the update events.
		/// Since now, the update events were ignored.
		/// </summary>
		public void SetFeedListener(IExternalFeedListener listener) {
			_listener= listener;

			foreach (ExternalFeedProducer myProducer in _statusGenerators.Values) {
				myProducer.SetFeedListener(listener);
			}
		}

		/// <summary>
		/// Forces sending an event with a full snapshot for a stock.
		/// </summary>
		public void SendCurrentValues(string itemName) {
			ExternalFeedProducer myProducer= (ExternalFeedProducer) _statusGenerators[itemName];
			if (myProducer == null) return;

			lock (_snapshotQueue) {
				_snapshotQueue.Add(myProducer);
				Monitor.Pulse(_snapshotQueue);
			}
		}
	}

	public class ExternalFeedProducer {
		public string _itemName;
		private int _status;
		private double _mean, _stddev;
		private string _equipName;
		
		private Random _random;
		private bool _haveNextNextGaussian;
		private double _nextNextGaussian;

		private IExternalFeedListener _listener;
		private Thread _thread;

		/// <summary>
		/// Initializes stock data based on the already prepared values. 
		/// </summary>
		public ExternalFeedProducer(string name, 
			int status, 
			double updateTimeMean, double updateTimeStdDev, string equipName) {
			_itemName = name;
			_status = status;
			_mean = updateTimeMean;
			_stddev = updateTimeStdDev;
			_equipName = equipName;
			
			_random = new Random();
			_haveNextNextGaussian= false;
			_nextNextGaussian= 0.0;

			ComputeNewValues();
		}

		public string GetItemName() {
			return _itemName;
		}

		public void SetFeedListener(IExternalFeedListener listener) {
			lock (this) {
				_listener = listener;
			}
		}

		public void Start() {
			lock (this) {
				if (_thread != null) return;

				_thread = new Thread(new ThreadStart(Run));
				_thread.Start();
			}
		}

		private void Run() {
			do {
				int waitMillis= ComputeNextWaitTime();
				Thread.Sleep(waitMillis);

				ComputeNewValues();
				if (_listener != null) 
					_listener.OnEvent(_itemName, GetCurrentValues(false), false);

			} while (true);
		}

		/// <summary>
		/// Decides, for ease of simulation, the time at which the next
		/// update for the stock will happen.
		/// </summary>
		public int ComputeNextWaitTime() {
			lock (this) {
				int millis;
				do {
					millis = (int) Gaussian(_mean, _stddev);
				} while (millis <= 0);
				return millis;
			}
		}

		/// <summary>
		/// Changes the current data for the stock. This stuff is to ensure that new prices follow a random
        /// but nondivergent path, centered around the reference price
		/// </summary>
		public void ComputeNewValues() {
			lock (this) {
			    if (_status == 1)
			    {
                    _status = 2;
			    } 
			    else if (_status == 2)
			    {
                    _status = 3;
			    }
			    else if (_status == 3)
			    {
                    _status = 1;
			    }
			}
		}

		/// <summary>
		/// Picks the stock field values and stores them in a field/value
		/// Hashtable. If fullData is false, then only the fields whose value
		/// is just changed are considered (though this check is not strict).
		/// </summary>
		public IDictionary GetCurrentValues(bool fullData) {
			lock (this) {
				IDictionary eventData = new Hashtable();
                eventData["equipName"] = _equipName;
                eventData["status"] = _status.ToString();
				
				return eventData;
			}
		}

		private double Gaussian(double mean, double stddev) {
			lock (this) {
				double b= 0.0;
				
				if (_haveNextNextGaussian) {
					_haveNextNextGaussian = false;
					b= _nextNextGaussian;
				
				} 
				else {
					double v1, v2, s;
					do { 
						v1 = 2.0 * _random.NextDouble() - 1.0;
						v2 = 2.0 * _random.NextDouble() - 1.0;
						s = v1 * v1 + v2 * v2;
					} while (s >= 1.0 || s == 0.0);
					double multiplier = Math.Sqrt(-2.0 * Math.Log(s)/s);
					_nextNextGaussian = v2 * multiplier;
					_haveNextNextGaussian = true;
					b= v1 * multiplier;
				}

				double val= b * stddev + mean;
				return val;
			}
		}

	}

}
